-----------------------------------------------------------------------------------------------
-- Client Lua Script for MobStats
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- MobStats Module Definition
-----------------------------------------------------------------------------------------------
local MobStats = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function MobStats:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
    return o
end

function MobStats:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- MobStats OnLoad
-----------------------------------------------------------------------------------------------
function MobStats:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("MobStats.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- MobStats OnDocLoaded
-----------------------------------------------------------------------------------------------
function MobStats:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "StatsForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		local wndMainLocation = WindowLocation.new(self.savedLocation)
		 if wndMainLocation then 
			if self.savedLocation and self.savedLocation.nOffsets[1] ~= 0 then
				self.wndMain:MoveToLocation(wndMainLocation)
				self.savedLocation = nil 
			end
    	end
		self.wndMain:Show(self.visibility)
		
		self.statLabel = {}
		self.statLabel[1] = self.wndMain:FindChild("Stat1"):FindChild("Label")
		self.statLabel[2] = self.wndMain:FindChild("Stat2"):FindChild("Label")
		self.statLabel[3] = self.wndMain:FindChild("Stat3"):FindChild("Label")
		self.statLabel[4] = self.wndMain:FindChild("Stat4"):FindChild("Label")
		self.statLabel[5] = self.wndMain:FindChild("Stat5"):FindChild("Label")
		self.statLabel[6] = self.wndMain:FindChild("Stat6"):FindChild("Label")
		self.statLabel[7] = self.wndMain:FindChild("Stat7"):FindChild("Label")
		self.statLabel[8] = self.wndMain:FindChild("Stat8"):FindChild("Label")
		self.statLabel[9] = self.wndMain:FindChild("Stat9"):FindChild("Label")
		self.statLabel[10] = self.wndMain:FindChild("Stat10"):FindChild("Label")
		self.statLabel[11] = self.wndMain:FindChild("Stat11"):FindChild("Label")
		self.statLabel[12] = self.wndMain:FindChild("Stat12"):FindChild("Label")
		self.statValue = {}
		self.statValue[1] = self.wndMain:FindChild("Stat1"):FindChild("Value")
		self.statValue[2] = self.wndMain:FindChild("Stat2"):FindChild("Value")
		self.statValue[3] = self.wndMain:FindChild("Stat3"):FindChild("Value")
		self.statValue[4] = self.wndMain:FindChild("Stat4"):FindChild("Value")
		self.statValue[5] = self.wndMain:FindChild("Stat5"):FindChild("Value")
		self.statValue[6] = self.wndMain:FindChild("Stat6"):FindChild("Value")
		self.statValue[7] = self.wndMain:FindChild("Stat7"):FindChild("Value")
		self.statValue[8] = self.wndMain:FindChild("Stat8"):FindChild("Value")
		self.statValue[9] = self.wndMain:FindChild("Stat9"):FindChild("Value")
		self.statValue[10] = self.wndMain:FindChild("Stat10"):FindChild("Value")
		self.statValue[11] = self.wndMain:FindChild("Stat11"):FindChild("Value")
		self.statValue[12] = self.wndMain:FindChild("Stat12"):FindChild("Value")
				
		for i=1, #self.statLabel do
			self.statLabel[i]:SetText("")
		end
		
		for i=1, #self.statValue do
			self.statValue[i]:SetText("")
		end

		self.unit = nil
		self.unitPlayer = GameLib.GetPlayerUnit()
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("mstat", "OnMobStatsToggle", self)
		Apollo.RegisterEventHandler("TargetUnitChanged", "OnTargetUnitChanged", self)
		Apollo.RegisterEventHandler("ChangeWorld", "ZoneCheck", self)
    	Apollo.RegisterEventHandler("SubZoneChanged", "ZoneCheck", self)
		Apollo.RegisterTimerHandler("MobStatsZoneTimer", "ZoneCheck", self)
	end
end

-----------------------------------------------------------------------------------------------
-- MobStats Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/mstat"
function MobStats:OnMobStatsToggle()
	self.visibility = not self.visibility
	self.wndMain:Show(self.visibility)
end

function MobStats:OnTargetUnitChanged(unit)
	self.unit = unit
	if unit then
		self:DimText(false)
	else
		self:DimText(true)
		self.checkTimer = nil
	end
	self:UpdateStats()

end

function MobStats:Round1(t)
	if t == nil then return " ---" end
	return tonumber(string.format("%.1f", t))
	
end

function MobStats:Round2(t)
	if t == nil then return " ---" end
	return tonumber(string.format("%.2f", t))	
end

function MobStats:AbbrNum(t)
	if t == nil then return end
	if t > 1000000 then
		t = t/1000000
		t = self:Round1(t)
		return t .. "m"
	elseif t > 1000 then
		t = t/1000
		t = self:Round1(t)
		return t .. "k"
	else
		return t
	end
end

function MobStats:UpdateStats()
	if 	self.unitPlayer == nil then
		self.unitPlayer = GameLib.GetPlayerUnit()
		return
	end
	if 	self.unit then
		self.statLabel[1]:SetText("")
		self.statValue[1]:SetText(self.unit:GetName())
		self.statLabel[2]:SetText("ID:")
		self.statValue[2]:SetText(self.unit:GetId())
		self.statLabel[3]:SetText("Health:")
		self.statValue[3]:SetText(self:AbbrNum(self.unit:GetHealth())) --.. "/".. self:AbbrNum(self.unit:GetMaxHealth()))
		self.statLabel[4]:SetText("Crit Chance:")
		self.statLabel[5]:SetText("Crit Sev:")
		self.statLabel[6]:SetText("Deflect:")
		self.statLabel[7]:SetText("Mitigation:")
		if self.unit:IsACharacter() and not self.unit:IsThePlayer() then
			self.statValue[4]:SetText(" --- ")
			self.statValue[5]:SetText(" --- ")
			self.statValue[6]:SetText(" --- ")
			self.statValue[7]:SetText(" --- ")
		else
			self.statValue[4]:SetText(self:Round2(self.unit:GetCritChance().nAmount))
			self.statValue[5]:SetText(self:Round2(self.unit:GetCritSeverity().nAmount))
			local d1 = self:Round2(self.unit:GetDeflectChance().nAmount)
			local d2 = 0
			if d1 >= 7.5 then
				d2 = d1 - 7.5
			end
			self.statValue[6]:SetText(d1 .. " ("..d2..")")
			self.statValue[7]:SetText(self:Round2(self.unit:GetPhysicalMitigation().nAmount))

		end
		self.statLabel[8]:SetText("Assault:")
		self.statValue[8]:SetText(self:Round2(self.unit:GetAssaultPower()))
		self.statLabel[9]:SetText("Level:")
		self.statValue[9]:SetText(self.unit:GetLevel())
		self.statLabel[10]:SetText("Eliteness:")
		self.statValue[10]:SetText(self.unit:GetEliteness())
		self.statLabel[11]:SetText("Gender:")
		local gender = self.unit:GetGender()
		if gender == 0 then
			self.statValue[11]:SetText("Male")
		elseif gender == 1 then
			self.statValue[11]:SetText("Female")
		else
			self.statValue[11]:SetText("Undefined")
		end
		self.statLabel[12]:SetText("Distance:")
		self.statValue[12]:SetText(self:GetDistanceTo(self.unit))
		if self.unit:IsACharacter() and not self.unit:IsThePlayer() then
			self.statValue[10]:SetText("Player")
		elseif self.unit:IsThePlayer() then
			self.statValue[10]:SetText("Player")
		end
		--GameLib.GetTargetUnit:	
		self.checkTimer = ApolloTimer.Create(0.5, false, "UpdateStats", self)
	else
		self.checkTimer = nil
		self.statValue[12]:SetText("???")
	end
end

function MobStats:DimText(dim)
	local color = "FFFFFFFF"
	if dim then
		color = "7FFFFFFF"
	end
	for i=1, 12 do
		self.statLabel[i]:SetTextColor(color)
		self.statValue[i]:SetTextColor(color)
	end
end

function MobStats:ZoneCheck()
	self:DimText(true)
	self.statValue[12]:SetText("???")
	self.checkTimer = nil
	self.unit = nil
	self.unitPlayer = GameLib.GetPlayerUnit()
	if not self.unitPlayer then
		Apollo.CreateTimer("MobStatsZoneTimer", 1, false)
	end
end

function MobStats:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	local tsave = {}
	tsave.version = 1.2
	tsave.savedLocation = self.wndMain:GetLocation():ToTable()
	tsave.visibility = self.visibility
	return tsave
end


function MobStats:OnRestore(eLevel, saveData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	self.version = saveData.version
	if self.version ~= 1.2 then
		self.savedLocation = nil
		self.visibility = true
	else
		self.savedLocation = saveData.savedLocation
		self.visibility = saveData.visibility
	end

end

function MobStats:GetDistanceTo(uUnit)
	if self.unitPlayer ~= nil then
		local tPos1 = self.unitPlayer :GetPosition()
		local tPos2 = uUnit:GetPosition()
		if tPos1 ~= nil and tPos2 ~= nil then
			return MobStats:Round2(math.sqrt(math.pow(tPos1.x - tPos2.x, 2) + math.pow(tPos1.y - tPos2.y, 2) + math.pow(tPos1.z - tPos2.z, 2)))
		end
	end
	return 99999
end

-----------------------------------------------------------------------------------------------
-- MobStats Instance
-----------------------------------------------------------------------------------------------
local MobStatsInst = MobStats:new()
MobStatsInst:Init()



